<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="utf-8" />
  <title>{% block title %}Grip{% endblock %}</title>
  <link rel="icon" href="{{ favicon or url_for('static', filename='favicon.ico') }}" />
  {%- block styles %}{% endblock %}
</head>
<body>
  <div class="page">
    {% block page %}{% endblock %}
  </div>
  {%- block scripts %}{% endblock %}
</body>
</html>
